#ifndef __MCTD_H__
#define __MCTD_H__

#include <stdbool.h>

#define STARTUP_CONFIG_PATH "/etc/aiot-modem-control/startup.conf"

#define DAEMON_PATH "/var/run/aiot-modem-controld/"
#define SOCK_PATH DAEMON_PATH "socket"
#define QUIT_PATH DAEMON_PATH "quit"
#define PID_PATH DAEMON_PATH "aiot-modem-controld.pid"

#define UNUSED(x) ((void)(x))
#define __unused __attribute__((unused))

#define MCTD_HEADER "mctd"

#define MCTD_CMD_STR_SET_APN "set-apn"
#define MCTD_CMD_STR_DIAL "dial"
#define MCTD_CMD_STR_HANGUP "hangup"
#define MCTD_CMD_STR_GET_PHONE_NUMBER "get-phone-number"
#define MCTD_CMD_STR_GET_SIGNAL_QUALITY "get-signal-quality"
#define MCTD_CMD_STR_WWAN_FORCE_RESTART "wwan-force-restart"
#define MCTD_CMD_STR_POWERON "poweron"
#define MCTD_CMD_STR_POWEROFF "poweroff"
#define MCTD_CMD_STR_SET_PSM "set-psm"
#define MCTD_CMD_STR_SET_EDRX "set-edrx"
#define MCTD_CMD_STR_SET_SUSPEND "set-suspend"
#define MCTD_CMD_STR_SET_SLEEP "set-sleep"
#define MCTD_CMD_STR_ACTIVATE "activate"
#define MCTD_CMD_STR_DEACTIVATE "deactivate"
#define MCTD_CMD_STR_SEND_AT "send-at"
#define MCTD_CMD_STR_SEND_AT_ECHO "send-at-echo"
#define MCTD_CMD_STR_SET_SIM_SLOT "set-sim-slot"
#define MCTD_CMD_STR_UPDATE_DATETIME "update-datetime"
#define MCTD_CMD_STR_GET_DATETIME "get-datetime"
#define MCTD_CMD_STR_POWEROFF_AND_FIN "poweroff-and-fin"
#define MCTD_CMD_STR_FIN "fin"
#define MCTD_CMD_STR_STATUS "status"
#define MCTD_CMD_STR_LIST_SMS "list-sms"
#define MCTD_CMD_STR_SHOW_SMS "show-sms"
#define MCTD_CMD_STR_CREATE_SMS "create-sms"
#define MCTD_CMD_STR_SEND_SMS "send-sms"
#define MCTD_CMD_STR_DELETE_SMS "delete-sms"
#define MCTD_CMD_STR_SET_SMS_STORAGE "set-sms-storage"

#define MCTD_STR_ENABLE "enable"
#define MCTD_STR_DISABLE "disable"
#define MCTD_STR_DEFAULT "default"
#define MCTD_SPOW_MIN_MSEC 100
#define MCTD_SPOW_MAX_MSEC 5000
#define MCTD_PSM_MIN_BIT 1
#define MCTD_PSM_MAX_BIT 31
#define MCTD_PSM_2SEC 2
#define MCTD_PSM_30SEC 30
#define MCTD_PSM_1MIN 1
#define MCTD_PSM_DECIHOURS 6
#define MCTD_PSM_10MIN 10
#define MCTD_PSM_1H 1
#define MCTD_PSM_10H 10
#define MCTD_AT_BUF_SIZE 185

#define MCTD_CHECK_REGISTER_INTERVAL_SEC_MIN 30
#define MCTD_CHECK_REGISTER_INTERVAL_SEC_MAX 300

typedef struct {
    char protocol_id[4];
} MCTD_HEADER_T;

typedef struct {
    unsigned char cmd_id[4];
    unsigned char param_len[4];
} MCTD_REQUEST_T;

typedef struct {
    unsigned char cmd_id[4];
    unsigned char result[4];
    unsigned char param_len[4];
} MCTD_RESPONSE_T;

enum MCTD_CMD_T {
    MCTD_CMD_NOP,
    MCTD_CMD_SET_APN,
    MCTD_CMD_DIAL,
    MCTD_CMD_HANGUP,
    MCTD_CMD_GET_PHONE_NUMBER,
    MCTD_CMD_GET_SIGNAL_QUALITY,
    MCTD_CMD_WWAN_FORCE_RESTART,
    MCTD_CMD_POWERON,
    MCTD_CMD_POWEROFF,
    MCTD_CMD_SET_PSM,
    MCTD_CMD_SET_EDRX,
    MCTD_CMD_SET_SUSPEND,
    MCTD_CMD_SET_SLEEP,
    MCTD_CMD_ACTIVATE,
    MCTD_CMD_DEACTIVATE,
    MCTD_CMD_SEND_AT,
    MCTD_CMD_SEND_AT_ECHO,
    MCTD_CMD_SET_SIM_SLOT,
    MCTD_CMD_UPDATE_DATETIME,
    MCTD_CMD_GET_DATETIME,
    MCTD_CMD_POWEROFF_AND_FIN,
    MCTD_CMD_FIN,
    MCTD_CMD_STATUS,
    MCTD_CMD_LIST_SMS,
    MCTD_CMD_SHOW_SMS,
    MCTD_CMD_CREATE_SMS,
    MCTD_CMD_SEND_SMS,
    MCTD_CMD_DELETE_SMS,
    MCTD_CMD_SET_SMS_STORAGE,
};

enum MCTD_RESULT_T {
    MCTD_RESULT_OK = 0,
    MCTD_RESULT_OK_ALREADY,
    MCTD_RESULT_ERR_CALLING,
    MCTD_RESULT_ERR_PROCESS,
    MCTD_RESULT_ERR_SOCKET,
    MCTD_RESULT_ERR_CMD_NOT_FOUND,
    MCTD_RESULT_ERR_MEMORY,
    MCTD_RESULT_ERR_USING_SERIAL,
    MCTD_RESULT_ERR_SERIAL,
    MCTD_RESULT_ERR_TIMEOUT,
    MCTD_RESULT_ERR_PARAM,
    MCTD_RESULT_ERR_NO_DATA,
    MCTD_RESULT_ERR_NO_SIM,
    MCTD_RESULT_ERR_NOT_REGISTERED,
    MCTD_RESULT_ERR_TERMINATED,
};

unsigned int mctd_u8tou32(unsigned char *u8);
void mctd_u32tou8(unsigned int u32, unsigned char *u8);
bool mctd_is_valid_header(const char *header);
int mctd_wait_data(int fd, int timeout_sec);
int mctd_wait_data_with_usec(int fd, int timeout_sec, int timeout_usec);
int mctd_receive_data(int fd, char* buf, int desire_len, int retry);

void mctd_daemon_quit();
void mctd_remove_quit_file();
bool mctd_is_daemon_quit();

bool mctd_create_pid_file();
void mctd_remove_pid_file();
bool mctd_has_pid_file();

#endif /* __MCTD_H__ */
